#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "shell.h"

/*
  dioctl.config contains lines which match one of the following patterns:

  1. NAME=# , where NAME does not contain any periods
  2. attribute.Connector.Count=#
  3. attrib.Connector.Name._=#
  4. attrib.Connector.#1.Pins=#2
  5. attrib.Connector.#1.#2=#3
  9. attrib.DIO.Count=#
  10. attrib.DIO.Instances=#

  6. ts.socket.*=#

  As we parse through our dioctl.config files, we have to keep track of
  the following:
  7. the current starting DIO number
     starts at 0
     increment by #9 * #10
  8. the current starting connector number
     starts at 1
     increments by the value of #2

  We have to modify entries as follows:
  #1: add #7 to #
  #3: add #8 to #
  #4: add #8 to #1
  #5: add #8 to #1, add #7 to #3

  if attrib.DIO.Instances is defined and # > 1, then we need to replicate
  the entries and follow the above for each additional entry.  We also need
  to give each #3 a unique name based on the original name plus the instance.
  e.g. PORTA_ -> PORTA_1_
 */
int line_type(char *line) {
  //3. attrib.Connector.Name._=#
  if (grep1(line,"attrib\\.Connector\\.Name\\.[A-Za-z_0-9]\\+=[0-9]\\+")) return 3;
  //2. attribute.Connector.Count=#
  if (grep1(line,"attrib\\.Connector\\.Count=[0-9]\\+")) return 2;
  //4. attrib.Connector.#1.Pins=#2
  if (grep1(line,"attrib\\.Connector\\.[0-9]\\+\\.Pins=[0-9]\\+")) return 4;
  //5. attrib.Connector.#1.#2=#3
  if (grep1(line,"attrib\\.Connector\\.[0-9]\\+\\.[0-9]\\+=[0-9]\\+")) return 5;
  //9. attrib.DIO.Count=#
  if (grep1(line,"attrib\\.DIO\\.Count=[0-9]\\+")) return 9;
  //10. attrib.DIO.Instances=#
  if (grep1(line,"attrib\\.DIO\\.Instances=[0-9]\\+")) return 10;
  //1. NAME=# , where NAME does not contain any periods
  if (grep1(line,"[A-Za-z0-9_]\\+=[0-9]\\+")) return 1;
  if (line[0] && line[0] != '#') {
    fprintf(stderr,"Warning:no match for '%s'\n",line);
  }
  return 0;
}

// X=# -> X=#+n
char *add(char *line,int n) {
  line = strdup(line);
  char *str = line,*ret;
  int num;

  ret = malloc(strlen(line)+2);
  while (*str != '=') str++;
  *str = 0;
  str++;
  num = strtoul(str,0,0);
  sprintf(ret,"%s=%d",line,num+n);
  return ret;
}

// X.STR=# -> X.STRz_=#
char *addN(char *line,int n,int z) {
  line = strdup(line);
  char *str = line,*ret;
  int num;

  ret = malloc(strlen(line)+10);
  while (*str != '=') str++;
  *str = 0;
  str++;
  num = strtoul(str,0,0);
  sprintf(ret,"%s%d_=%d",line,z,num+n);
  return ret;
}

// X.#1.Pins=#2 -> X.#1+n.Pins=#2
char *add1(char *line,int n) {
  line = strdup(line);
  char *str = line,*ret;
  int num;

  ret = malloc(strlen(line)+2);
  while (!isdigit(*str)) str++;
  num = atoi(str);
  *str = 0;
  while (*str != '.') str++;
  sprintf(ret,"%s%d%s",line,num+n,str);
  return ret;  
}

// X.#1.#2=#3 -> X.#1+n.#2=#3+m
char *add13(char *line,int n,int m) {
  line = strdup(line);
  char *str = line,*str2,*ret;
  int num,num2;

  ret = malloc(strlen(line)+4);
  while (!isdigit(*str)) str++;
  num = atoi(str);
  *str = 0;
  while (*str != '.') str++;
  str2 = str;
  while (*str2 != '=') str2++;
  *str2 = 0;
  str2++;
  num2 = atoi(str2);
  sprintf(ret,"%s%d%s=%d",line,num+n,str,num2+m);
  return ret;
}

// X=#, return #
int value(char *line) {
  char *str = line;
  while (*str != '=') str++;
  str++;
  return strtol(str,0,0);
}

int main(int argc,char *argv[],char **env) {
  int i,configlen,j,type,thisInstance=1;
  int DIOStart = 0,DIOConn = 0, DIOInstances = 1;
  int DIOCount,DIOConns,inst;
  char **config,*str;

  for (i=1;i<argc;i++) {
    config = cat(argv[i]);
    configlen = len(config);
    DIOCount = 0;
    DIOConns = 0;
    inst = -1;
    for (thisInstance=1;thisInstance <= DIOInstances;thisInstance++) {
      for (j=0;j<configlen;j++) {
	type = line_type(config[j]);
	str = config[j];
	switch (type) {
	case 1: str = add(config[j],DIOStart); break;
	case 2: DIOConns = value(config[j]); break;
	case 3: 
	  if (inst == -1) {
	    str = add(config[j],DIOConn); break;
	  } else {
	    str = addN(config[j],DIOConn,thisInstance); break;
	  }
	case 4: str = add1(config[j],DIOConn); break;
	case 5: str = add13(config[j],DIOConn,DIOStart); break;
	case 9: DIOCount = value(config[j]); break;
	case 10: 
	  if (thisInstance == 1) {
	    inst = DIOInstances = value(config[j]); break;
	  }
	}
	if (type != 9 && type != 10 && type != 2) {
	  printf("%s\n",str);
	}
      }
      DIOStart += DIOCount;
      DIOConn += DIOConns;
    }
  }
  printf("attrib.Connector.Count=%d\n",DIOConn);
  printf("attrib.DIO.Count=%d\n",DIOStart);
}
//gcc -g -o native/config-gen config-gen.c HashTable.c shell.c
